/* 
 * Copyright (C) 2001 Jan Outrata <outrataj@phoenix.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdlib.h>
#include <math.h>
#include "graphics.h"
#include "screen.h"

struct line_2d_t *first_line = NULL;
struct poly_2d_t *first_poly = NULL;

void
point(struct point_2d_t *p, long int c)
{
  screen_buffer[p->y][p->x] = c;
}

long int
point_get(struct point_2d_t *p)
{
  return screen_buffer[p->y][p->x];
}

struct line_2d_t *
add_line(struct line_2d_t *line)
{
  struct line_2d_t *l, *last;

  l = (struct line_2d_t *)malloc(sizeof(struct line_2d_t));
  if (l == NULL) return NULL;
  *l = *line; l->next = NULL;
  if (first_line == NULL) { l->id = 0; first_line = l; }
  else {
    last = first_line;
    while (last->next != NULL) last = last->next;
    l->id = last->id + 1; last->next = l;
  }
  return l;
}

void
line_dda(struct line_2d_t *line, long int c, 
	 long int (* f)(struct point_2d_t *, long int))
{
  int x, x1, y1, x2, y2;
  float y, s;
  struct point_2d_t p;

  x1 = line->points[0].x; y1 = line->points[0].y; 
  x2 = line->points[1].x; y2 = line->points[1].y;
  if (x1 > x2) { 
    x2 = line->points[0].x; y2 = line->points[0].y; 
    x1 = line->points[1].x; y1 = line->points[1].y;
  }
  if (x1 != x2) s = (float)(y2-y1)/(x2-x1); else s = 2;
  if (fabs(s) <= 1) {
    y = y1;
    for (x = x1; x <= x2; ++x) { 
      p.x = x; p.y = (int)rint(y);
      if (f == NULL) point(&p, c); else point(&p, f(&p, c));
      y += s;
    }
  }
  else {
    x1 = line->points[0].x; y1 = line->points[0].y; 
    x2 = line->points[1].x; y2 = line->points[1].y;
    if (y1 > y2) { 
      x2 = line->points[0].x; y2 = line->points[0].y; 
      x1 = line->points[1].x; y1 = line->points[1].y;
    }
    if (x1 != x2) s = 1/s; else s = 0;
    y = x1;
    for (x = y1; x <= y2; ++x) {
      p.x = (int)rint(y); p.y = x;
      if (f == NULL) point(&p, c); else point(&p, f(&p, c));
      y += s;
    }
  }
}

void
line_bres(struct line_2d_t *line, long int c, 
	  long int (* f)(struct point_2d_t *, long int))
{
  int x, y, x1, y1, x2, y2, s, dx, dy, incdx, incdy, i;
  struct point_2d_t p;

  x1 = line->points[0].x; y1 = line->points[0].y; 
  x2 = line->points[1].x; y2 = line->points[1].y;
  if (x1 > x2) { 
    x2 = line->points[0].x; y2 = line->points[0].y; 
    x1 = line->points[1].x; y1 = line->points[1].y;
  }
  if (abs(x2-x1) >= abs(y2-y1)) {
    x = x1; y = y1; s = x2-x1; dx = 2*s; dy = 2*abs(y2-y1);
    incdy = ((y2-y1) > 0) ? 1 : -1;
    for (i = x1; i <= x2; ++i) { 
      p.x = x; p.y = y;
      if (f == NULL) point(&p, c); else point(&p, f(&p, c));
      ++x;
      s -= dy;
      if (s < 0) { y += incdy; s += dx; }
    }
  }
  else {
    x1 = line->points[0].x; y1 = line->points[0].y; 
    x2 = line->points[1].x; y2 = line->points[1].y;
    if (y1 > y2) { 
      x2 = line->points[0].x; y2 = line->points[0].y; 
      x1 = line->points[1].x; y1 = line->points[1].y;
    }
    x = x1; y = y1; s = y2-y1; dx = 2*abs(x2-x1); dy = 2*s;
    incdx = ((x2-x1) > 0) ? 1 : -1;
    for (i = y1; i <= y2; ++i) { 
      p.x = x; p.y = y;
      if (f == NULL) point(&p, c); else point(&p, f(&p, c));
      ++y;
      s -= dx;
      if (s < 0) { x += incdx; s += dy; }
    }
  }
}

void
poly_2d_init(struct poly_2d_t *poly)
{
  poly->n = 0;
  poly->points = NULL;
}

void
poly_2d_fini(struct poly_2d_t *poly)
{
  poly->n = 0;
  free(poly->points); poly->points = NULL;
}

int
poly_2d_add_point(struct poly_2d_t *poly, struct point_2d_t *point)
{
  struct point_2d_t *new;
  int i, n;
  
  n = poly->n;
  new = (struct point_2d_t *)malloc((n+1)*sizeof(struct point_2d_t));
  if (new == NULL) return -1;
  for (i = 0; i < n; ++i) new[i] = poly->points[i];
  new[n] = *point;
  poly->n++;
  free(poly->points); poly->points = new;
  return 0;
}

int
poly_2d_del_point(struct poly_2d_t *poly)
{
  struct point_2d_t *new;
  int i, n;
  
  n = poly->n;
  new = (struct point_2d_t *)malloc((n-1)*sizeof(struct point_2d_t));
  if (new == NULL) return -1;
  for (i = 0; i < n-1; ++i) new[i] = poly->points[i];
  poly->n--;
  free(poly->points); poly->points = new;
  return 0;
}

void
poly_2d_draw(struct poly_2d_t *poly, long int c)
{
  int i;
  struct line_2d_t line;
  
  for (i = 0; i < poly->n; ++i) {
    line.points[0] = poly->points[i];
    line.points[1] = poly->points[(i+1) % poly->n];
    line_bres(&line, c, NULL);
  }
}

struct poly_2d_t *
add_poly(struct poly_2d_t *poly)
{
  struct poly_2d_t *p, *last;
  struct point_2d_t *ps;
  int i = 0;

  p = (struct poly_2d_t *)malloc(sizeof(struct poly_2d_t));
  if (p == NULL) return NULL;
  *p = *poly;
  ps = (struct point_2d_t *)malloc(poly->n * sizeof(struct point_2d_t));
  for (; i < poly->n; ++i) ps[i] = poly->points[i];
  p->points = ps;
  p->next = NULL;
  if (first_poly == NULL) { p->id = 0; first_poly = p; }
  else {
    last = first_poly;
    while (last->next != NULL) last = last->next;
    p->id = last->id + 1; last->next = p;
  }
  return p;
}

void 
fill_seed4(struct point_2d_t *p, long int c)
{
  long int old;
  struct point_2d_t p1;

  if ((p->x >= 0) && (p->x < SCREEN_WIDTH) && (p->y >= 0) && 
      (p->y < SCREEN_HEIGHT)) {  
    old = point_get(p);
    point(p, c);

    p1.x = p->x-1; p1.y = p->y;
    if ((p1.x >= 0) && (point_get(&p1) == old)) fill_seed4(&p1, c);
    p1.x += 2;
    if ((p1.x < SCREEN_WIDTH) && (point_get(&p1) == old)) fill_seed4(&p1, c);
    p1.x -= 1; p1.y -= 1;
    if ((p1.y >= 0) && (point_get(&p1) == old)) fill_seed4(&p1, c);
    p1.y += 2;
    if ((p1.y < SCREEN_HEIGHT) && (point_get(&p1) == old)) fill_seed4(&p1, c);
  }

}

long int
xor(struct point_2d_t *p, long int c)
{
  struct point_2d_t p1;

  for (p1 = *p; p1.x < SCREEN_WIDTH; ++(p1.x)) point(&p1, point_get(&p1) ^ c);
  return point_get(p) ^ c;
}

void
line_bres_y_step(struct line_2d_t *line, long int c, 
		 long int (* f)(struct point_2d_t *, long int))
{
  int x, y, x1, y1, x2, y2, s, dx, dy, incdx, incdy, i, n;
  struct point_2d_t p;

  x1 = line->points[0].x; y1 = line->points[0].y; 
  x2 = line->points[1].x; y2 = line->points[1].y;
  if (x1 > x2) { 
    x2 = line->points[0].x; y2 = line->points[0].y; 
    x1 = line->points[1].x; y1 = line->points[1].y;
  }
  if (abs(x2-x1) >= abs(y2-y1)) {
    x = x1; y = y1; s = x2-x1; dx = 2*s; dy = 2*abs(y2-y1);
    incdy = ((y2-y1) > 0) ? 1 : -1;
    n = 1;
    for (i = x1; i <= x2; ++i) { 
      if (n) {
	p.x = x; p.y = y;
	if (f == NULL) point(&p, c); else point(&p, f(&p, c));
      }
      ++x;
      s -= dy;
      if (s < 0) { y += incdy; s += dx; n = 1; } else n = 0;
    }
  }
  else {
    x1 = line->points[0].x; y1 = line->points[0].y; 
    x2 = line->points[1].x; y2 = line->points[1].y;
    if (y1 > y2) { 
      x2 = line->points[0].x; y2 = line->points[0].y; 
      x1 = line->points[1].x; y1 = line->points[1].y;
    }
    x = x1; y = y1; s = y2-y1; dx = 2*abs(x2-x1); dy = 2*s;
    incdx = ((x2-x1) > 0) ? 1 : -1;
    for (i = y1; i <= y2; ++i) { 
      p.x = x; p.y = y;
      if (f == NULL) point(&p, c); else point(&p, f(&p, c));
      ++y;
      s -= dx;
      if (s < 0) { x += incdx; s += dy; }
    }
  }
}

void
fill_inv(struct poly_2d_t *poly, long int c)
{
  int i;
  long int c1;
  struct line_2d_t line;
  
  if (poly->n == 0) return;
  c1 = point_get(poly->points);
  for (i = 0; i < poly->n; ++i) {
    // if (poly->points[i].y != poly->points[(i+1) % poly->n].y)
    line.points[0] = poly->points[i];
    line.points[1] = poly->points[(i+1) % poly->n];
    line_bres_y_step(&line, c, xor);
  }
  poly_2d_draw(poly, c1);
}

struct point_2d_t *
line_find_x(struct line_2d_t *line, int y)
{
  struct point_2d_t *p = NULL;

  p = (struct point_2d_t *)malloc(sizeof(struct point_2d_t));
  if (p == NULL) return NULL;
  p->y = y;
  p->x = 
    (int)(((float)((line->points[1].x - line->points[0].x)) / 
	   (line->points[1].y - line->points[0].y)) *
	  (y - line->points[0].y)) + line->points[0].x;
  return p;
}

struct point_2d_t *
line_find_y(struct line_2d_t *line, int x)
{
  struct point_2d_t *p = NULL;

  p = (struct point_2d_t *)malloc(sizeof(struct point_2d_t));
  if (p == NULL) return NULL;
  p->x = x;
  p->y = 
    (int)(((float)((line->points[1].y - line->points[0].y)) / 
	   (line->points[1].x - line->points[0].x)) *
	  (x - line->points[0].x)) + line->points[0].y;
  return p;
}

int
clip_line(struct line_2d_t *line, struct point_2d_t *min, 
	  struct point_2d_t *max)
{
  const char left = 1;
  const char right = 2;
  const char bottom = 4;
  const char top = 8;
  char p[2] = {0, 0};
  int i;
  struct point_2d_t *pt;

  for (i = 0; i < 2; ++i) {
    if (line->points[i].x < min->x) p[i] |= left;
    if (line->points[i].x > max->x) p[i] |= right;
    if (line->points[i].y < min->y) p[i] |= bottom;
    if (line->points[i].y > max->y) p[i] |= top;
  }
  if ((!p[0]) && (!p[1])) return 1;
  if (p[0] & p[1]) return 0;
  for (i = 0; i < 2; ++i)
    if (p[i] & left) {
      pt = line_find_y(line, min->x);
      if (pt != NULL) line->points[i] = *pt; else return 0;
      return clip_line(line, min, max);
    } else
      if (p[i] & right) {
	pt = line_find_y(line, max->x);
	if (pt != NULL) line->points[i] = *pt; else return 0;
	return clip_line(line, min, max);
      } else
	if (p[i] & bottom) {
	  pt = line_find_x(line, min->y);
	  if (pt != NULL) line->points[i] = *pt; else return 0;
	  return clip_line(line, min, max);
	} else
	  if (p[i] & top) {
	    pt = line_find_x(line, max->y);
	    if (pt != NULL) line->points[i] = *pt; else return 0;
	    return clip_line(line, min, max);
	  }
  return 0; // not reached
}

void
malir(struct poly_3d_t **polys, int pn)
{
  int i, j, max;
  struct poly_3d_t *p;
  for (i = 0; i < pn; ++i) {
    max = i;
    for (j = i+1; j < pn; ++j) if (polys[j]->z > polys[max]->z) max = j;
    p = polys[i];
    polys[i] = polys[max];
    polys[max] = p;
  }
}
