/* 
 * Copyright (C) 2001 Jan Outrata <outrataj@phoenix.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __GRAPHICS_H__
#define __GRAPHICS_H__

struct point_2d_t {
  int x;
  int y;
};

struct line_2d_t {
  struct point_2d_t points[2];
  int id;
  struct line_2d_t *next;
};

extern struct line_2d_t *first_line;

struct poly_2d_t {
  int n;
  struct point_2d_t *points;
  int id;
  struct poly_2d_t *next;
};

extern struct poly_2d_t *first_poly;

struct poly_3d_t {
  struct poly_2d_t *poly_2d;
  int z;
};

void
point(struct point_2d_t *p, long int c);

long int
point_get(struct point_2d_t *p);

struct line_2d_t *
add_line(struct line_2d_t *line);

void
line_dda(struct line_2d_t *line, long int c, 
	 long int (* f)(struct point_2d_t *, long int));

void
line_bres(struct line_2d_t *line, long int c, 
	  long int (* f)(struct point_2d_t *, long int));

void
poly_2d_init(struct poly_2d_t *poly);

void
poly_2d_fini(struct poly_2d_t *poly);

int
poly_2d_add_point(struct poly_2d_t *poly, struct point_2d_t *point);

int
poly_2d_del_point(struct poly_2d_t *poly);

void
poly_2d_draw(struct poly_2d_t *poly, long int c);

struct poly_2d_t *
add_poly(struct poly_2d_t *poly);

void 
fill_seed4(struct point_2d_t *p, long int c);

long int
xor(struct point_2d_t *p, long int c);

void
line_bres_y_step(struct line_2d_t *line, long int c, 
		 long int (* f)(struct point_2d_t *, long int));

void
fill_inv(struct poly_2d_t *poly, long int c);

struct point_2d_t *
line_find_x(struct line_2d_t *line, int y);

struct point_2d_t *
line_find_y(struct line_2d_t *line, int x);

int
clip_line(struct line_2d_t *line, struct point_2d_t *min, 
	  struct point_2d_t *max);

void
malir(struct poly_3d_t **polys, int pn);

#endif
