/* 
 * Copyright (C) 2001 Jan Outrata <outrataj@phoenix.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdlib.h>
#include <stdio.h>
#include "screen.h"
#include "graphics.h"

extern long int **screen_buffer;

void
draw_console()
{
  int i, j;
  unsigned char c;

  if (screen_buffer == NULL) return;
  for (i = 0; i < 25; ++i) for (j = 0; j < 80; ++j) {
    c = ((screen_buffer[24-i][j] * 225) / (256 * 256 * 256)) + 32;
    printf("%c", (unsigned char)c);
  }
}

int
main ()
{
  int i, x, y;
  struct point_2d_t ps[10], p;
  struct poly_2d_t poly, window;
  struct line_2d_t l1, *l2 = NULL;
  struct poly_2d_t rects[2];
  struct poly_3d_t rects_3d[2];
  struct poly_3d_t *polys_3d[2];
  int ch;

  if (screen_buffer_init()) exit(1);
  screen_buffer_clear(rgb(0, 0, 0));

  printf("\n1 - line algorithm (Bresenham(x) or DDA)\n2 - fill algorithm (seed, inverse)\n3 - line clipping algorithm (Cohen-Suttherland)\n4 - poly visibility algorithm (painter)\n\n");
  do {
    printf("Your choice: ");
    scanf("%i", &ch);
  } while ((ch <= 0) || (ch > 4));

  if (ch < 4) {
    x = 40; y = 20;
    ps[0].x = 0+x; ps[0].y = 0+y;
    ps[1].x = -4+x; ps[1].y = -7+y;
    ps[2].x = -18+x; ps[2].y = -7+y;
    ps[3].x = -8+x; ps[3].y = -11+y;
    ps[4].x = -12+x; ps[4].y = -18+y;
    ps[5].x = 0+x; ps[5].y = -14+y;
    ps[6].x = 12+x; ps[6].y = -18+y;
    ps[7].x = 8+x; ps[7].y = -11+y;
    ps[8].x = 18+x; ps[8].y = -7+y;
    ps[9].x = 4+x; ps[9].y = -7+y;
    poly_2d_init(&poly);
    for (i = 0; i < 10; ++i) if (poly_2d_add_point(&poly, &ps[i])) exit(1);
    
    poly_2d_draw(&poly, rgb(100, 100, 100));
  }
  if (ch == 2) {
    p.x = 40; p.y = 10;
    fill_seed4(&p, rgb(200, 200, 200));
    draw_console();
    fill_inv(&poly, rgb(200, 200, 200) ^ rgb(150, 150, 150));
  }
  if (ch == 3) {
    x = 35; y = 5;
    ps[0].x = 0+x; ps[0].y = 0+y;
    ps[1].x = 10+x; ps[1].y = 0+y;
    ps[2].x = 10+x; ps[2].y = 10+y;
    ps[3].x = 0+x; ps[3].y = 10+y;
    poly_2d_init(&window);
    for (i = 0; i < 4; ++i) if (poly_2d_add_point(&window, &ps[i])) exit(1);

    poly_2d_draw(&window, rgb(50, 50, 50));
    
    for (i = 0; i < 10; ++i) {
      l1.points[0] = poly.points[i];
      l1.points[1] = poly.points[(i+1)%10];
      l2 = clip_line(&l1, &ps[0], &ps[2]);
      if (l2 != NULL) { line_bres(l2, rgb(70, 70, 70), NULL); free(l2); }
    }

    poly_2d_fini(&window);
  }
  if (ch < 4) poly_2d_fini(&poly);
  if (ch == 4) {
    x = 5; y = 5;
    ps[0].x = 0+x; ps[0].y = 0+y;
    ps[1].x = 50+x; ps[1].y = 0+y;
    ps[2].x = 50+x; ps[2].y = 10+y;
    ps[3].x = 0+x; ps[3].y = 10+y;
    poly_2d_init(&rects[0]);
    for (i = 0; i < 4; ++i) if (poly_2d_add_point(&rects[0], &ps[i])) exit(1);

    x = 20; y = 2;
    ps[0].x = 0+x; ps[0].y = 0+y;
    ps[1].x = 10+x; ps[1].y = 0+y;
    ps[2].x = 10+x; ps[2].y = 18+y;
    ps[3].x = 0+x; ps[3].y = 18+y;
    poly_2d_init(&rects[1]);
    for (i = 0; i < 4; ++i) if (poly_2d_add_point(&rects[1], &ps[i])) exit(1);

    rects_3d[0].poly_2d = &rects[0]; rects_3d[1].poly_2d = &rects[1];
    rects_3d[0].z = 10; rects_3d[1].z = 20;
    for(i = 0; i < 2; ++i) polys_3d[i] = &rects_3d[i];
    malir(polys_3d, 2);
    poly_2d_draw(polys_3d[0]->poly_2d, rgb(10, 10, 10));
    poly_2d_draw(polys_3d[1]->poly_2d, rgb(20, 20, 20));

    poly_2d_fini(&rects[0]);
    poly_2d_fini(&rects[1]);
  }

  draw_console();

  screen_buffer_fini();
  return 0;
}
