/* 
 * Copyright (C) 2001 Jan Outrata <outrataj@phoenix.inf.upol.cz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdlib.h>
#include "screen.h"

long int **screen_buffer = NULL;

int
screen_buffer_init()
{
  int i, j;
  
  screen_buffer = (long int **)malloc(SCREEN_HEIGHT * sizeof(long int *));
  if (screen_buffer == NULL) return -1;
  for (i = 0; i < SCREEN_HEIGHT; ++i) {
    screen_buffer[i] = (long int *)malloc(SCREEN_WIDTH * sizeof (long int));
    if (screen_buffer[i] == NULL) {
      for (j = i-1; j >= 0; --j) free(screen_buffer[j]);
      free(screen_buffer);
      return -1;
    }
  }
  return 0;
}

void
screen_buffer_fini()
{
  int i;

  if (screen_buffer == NULL) return;
  for (i = 0; i < SCREEN_HEIGHT; ++i) free(screen_buffer[i]);
  free(screen_buffer);
}

void
screen_buffer_clear(long int c)
{
  int i, j;

  if (screen_buffer == NULL) return;
  for (i = 0; i < SCREEN_HEIGHT; ++i) for (j = 0; j < SCREEN_WIDTH; ++j)
    screen_buffer[i][j] = c;
}

long int
rgb(unsigned char r, unsigned char g, unsigned char b)
{
  return (r * 256 * 256) + (g * 256) + b;
}

void
unrgb(long int c, unsigned char *r, unsigned char *g, unsigned char *b)
{
  *b = c % 256;
  c = (c - *b) / 256;
  *g = c % 256;
  *r = (c - *g) / 256;
}

int
check_screen_coords(int x, int y)
{
  if ((x >= 0) && (x < SCREEN_WIDTH) && (y >= 0) && (y < SCREEN_HEIGHT))
    return 1;
  else return 0;
}
